; $Id: $

	list	p=18f4550
	#include "p18f4550.inc"

; -----------------------

	; org 0x00 ~ 0x200 reserved for Jolt/Colt PIC18 serial bootloader.
	; Am I shelling out for an ICD2 hockey puck when I can bootload? Puck no, James!

; -----------------------
	org	0x200			; relocated reset vector
	goto	start

	org	0x208			; relocated interrupt vector
	goto	start

	org	0x218			; relocated interrupt vector
	goto	start
; -----------------------



#include	"string_m.inc"		; string / strcpy macro

start:
	movlw	B'11110000'			; set oscillator mode 8MHz, prefer external
	movwf	OSCCON				; ...
								; Really need the full 48MHz here, but at least will set the max INTOSC
								; here to (less likely) burn up the inverter circuit if the primary oscillator isn't running


	bcf		WDTCON, SWDTEN		; kill watchdog timer
	call	init_ports			; set up direction registers and A/D setting
	call	INITUSART			; set up USART (RS232) and baud rate

	call	delaybit			; still waiting for clocks to stabilize


test:
	 sendstring	str_ready		; show that the PIC serial port works and program is running...
								; Removed so I could cut the string macro and save a few bytes...







	bsf		LCDC, STB
	bsf		LCDC, SHDB
	bsf		LCDC, SHL

	bcf		RESETPORT, GRB		; GRB\ line to LCD. Can probably leave floating; there's RC reset logic on the PCB.
	call	delaysec
	bsf		RESETPORT, GRB

	bsf		LCDC, SEL0
	call	delaybit
	bcf		LCDC, SEL0



		; Every other line to the LCD must be complemented, and additionally, every other frame must be complemented again,
		; matching (presumably) the FRP signal. This program (and my board) doesn't poll FRP\, but it should be OK
		; as long as neither side loses count.
		;
		; To save cycles I reduced these to just inverting this single variable at the start of each line/frame, then 
		; XORing each byte with it before sending.


	setf	LINEMOD				; Modifier for line/field polarity.


		; The following constants set the V/H timings correctly to center the image on screen.
		; The timings in the datasheet were Greek to me (and didn't work); so these
		; were determined by trial and error.
		;
		; To play with them yourself, connect hyperterminal to the serial port (115200/8/N/1)
		; and use the +/- and number pad (arrow keys) to change the position and # blank lines.
		;
		; Note that where a 2-byte 'countdown' loop is used, the high byte has 1 added to it so we can terminate
		; the loop by comparing the high byte to 0. E.g. the 'real' horiz. length below is 118h.

	movlw	H'02'		; Number of valid data bytes (DCLKs) to send per line
	movwf	HLENH		;
	movlw	H'18'		;
	movwf	HLENL		;

	movlw	H'3c'		; Number of dummy DCLKs to send after dropping HSYNC at end of line
	movwf	HBLANKL		;

	movlw	H'79'		; Number of vertical lines to send
	movwf	VLENL		;



main:

sendscreen:
	movff	VLENL, LINECNTL ; 260 lines per screen; 220 valid

	bcf		LCDC, VSYNC	; VSYNC for a couple lines
	call	sendoddline	; ...
	call	sendoddline	;
	bsf		LCDC, VSYNC	; done VSYNC

	call	sendoddline
	call	sendoddline
	call	sendoddline
	call	sendoddline
	call	sendoddline
	call	sendoddline
	call	sendoddline
	call	sendoddline
	call	sendoddline
	call	sendoddline
	call	sendoddline
	call	sendoddline
	call	sendoddline
	call	sendoddline
	call	sendoddline
	call	sendoddline	; start on line 1 of LCD...



	clrf	TBLPTRU
	movlw	high	splash	; Point table index to start of image data
	movwf	TBLPTRH			; ...
	movlw	low		splash	;
	movwf	TBLPTRL			;




sendscreen2:
	call	sendevenline
	call	sendoddline

	call	ui_get_key	; debug - move screen position around by adjusting H/Vblank

	decfsz	LINECNTL	; done sending lines?
	goto	sendscreen2	; if no

	comf	LINEMOD		; switch frame polarity

	goto	sendscreen





; -----
sendevenline:
	comf	LINEMOD
	call	sendline
	return


sendoddline:
	comf	LINEMOD
	call	sendline
	return



sendline:

	movff	HLENH, BYTECNTH
	movff	HLENL, BYTECNTL

	bsf		LCDC, HSYNC		; HSYNC for 2 pixel clk before data starts
	bsf		LCDC, DCLK		; ...
	bcf		LCDC, DCLK		;
	bsf		LCDC, DCLK		;
	bcf		LCDC, DCLK		;


sendline2:

	TBLRD*+					; next byte from table
	movf	TABLAT, w		;
	xorwf	LINEMOD, w		; Correct for even/odd lines/frames (bleh)
	movwf	LCD				; onto port



	; --- inline 'clk'
	bsf		LCDC, DCLK
;	nop
	bcf		LCDC, DCLK
	; --- end inline 'clk'

	decfsz	BYTECNTL			; sent all bytes of line?
	goto	sendline2			; if no
	decfsz	BYTECNTH
	goto	sendline2

sendlinehblank:					; send the dummy clocks at end of line...
	clrf	LCD
	bcf		LCDC, HSYNC			; done HSYNC
	movff	HBLANKL, BYTECNTL

sendlinehblank2:
;	nop
;	nop
;	nop

	; --- inline 'clk'
	bsf		LCDC, DCLK
;	nop
	bcf		LCDC, DCLK
	; --- end inline 'clk'

	decfsz	BYTECNTL			; sent all bytes of line?
	goto	sendlinehblank2		; if no
;	decfsz	BYTECNTH
;	goto	sendlinehblank2

	return





stop:
	goto	stop




; ------------ arbitrary delay loops
delaysec:	; delay 1 second with 4MHz crystal
	movlw	17h		;17h
	movwf	del3
d3:
	movlw	80h
	movwf	del2
d2:
	movlw	70h
	movwf	del1
d1:
	decfsz	del1,f
	goto	d1

	decfsz	del2,f
	goto	d2

	decfsz	del3,f
	goto	d3

	retlw	0
; ------------------------

delaybit:
	;clrf waiter
	movlw	H'01'
	movwf	waiter
delayb1:	
	decfsz	waiter
	goto	delayb1

	return

; ------------------------

#include	"ports.inc"			; port initialization
#include	"4220uart.inc"		; RS232 stuff
#include	"lcmhex01.inc"		; Hex <--> ASCII conversion stuff (for debug)
#include	"defs.inc"			; Port and bit definitions
#include	"vars.inc"			; Variable (RAM) definitions
;#include	"sendbin.inc"		; Binary <--> ASCII conversion stuff (for debug)
;#include	"delay.inc"			; misc. delay loops
#include	"ui.inc"			; interactive screen-mover via RS232 commands
;#include	"linedata.inc"		; some junk for displaying
#include	"splash.inc"		; some junk for displaying


str_ready:
	DW	"Hello world! Sending things to Earth...\r\n\x00"

str_main_ver:
	DW	"$Id: $\r\n\x00"

	end
